@@ -16,6 +16,7 @@ import getpass
 import gnupg
 import hashlib
 import io
+import keyring
 import multiprocessing as mp
 import os
 import paramiko.ssh_exception as sshe
@@ -28,7 +29,7 @@ import tarfile
 import textwrap
 import uuid
 
-__version__ = "2.1.1"
+__version__ = "2.2.0"
 
 # Class Definitions
 
@@ -97,24 +98,29 @@ def build_ops_list(namespace):
                 access_test_results = access_test(absolute_path)
                 if False not in access_test_results:
                     size = os.path.getsize(absolute_path)
-                    if size <= ns.block_size_raw:  # We'll be handling this file.
-                        hasher = hashlib.new('sha256')
-                        with open(absolute_path, "rb") as contents:
-                            chunk = contents.read(io.DEFAULT_BUFFER_SIZE)
-                            while chunk != b"":
-                                hasher.update(chunk)
+                    try:
+                        if size <= ns.block_size_raw:  # We'll be handling this file.
+                            with open(absolute_path, "rb") as contents:
                                 chunk = contents.read(io.DEFAULT_BUFFER_SIZE)
-                        hash_digest = hasher.hexdigest()
-                        file_descriptor = {
-                            'fname': file, 'sha256': hash_digest, 'category': category,
-                            'fpath': sub_path, 'fsize': size
-                            }
-                        files_index.update({str(uuid.uuid1(node)): file_descriptor})
-                    else:
-                        size_pretty = size / 1048576
-                        block_size_pretty = ns.block_size_raw / 1048576
-                        message = ("{%s} %s is larger than %s (%s) and is being excluded" %
-                            (category, file, size_pretty, block_size_pretty))
+                                while chunk != b"":
+                                    hasher.update(chunk)
+                                    chunk = contents.read(io.DEFAULT_BUFFER_SIZE)
+                            hash_digest = hasher.hexdigest()
+                            file_descriptor = {
+                                'fname': file, 'sha256': hash_digest, 'category': category,
+                                'fpath': sub_path, 'fsize': size
+                                }
+                            files_index.update({str(uuid.uuid1(node)): file_descriptor})
+                        else:
+                            size_pretty = size / 1048576
+                            block_size_pretty = ns.block_size_raw / 1048576
+                            message = ("{%s} %s is larger than %s (%s) and is being excluded" %
+                                (category, file, size_pretty, block_size_pretty))
+                            print(message)
+                            ns.logs.log(message)
+                    except PermissionError:
+                        message = ("Error accessing %s: %s. Was this a network share file?"
+                                   % (absolute_path, access_test_results))
                         print(message)
                         ns.logs.log(message)
                 else:
@@ -534,12 +540,23 @@ def generate_keys(namespace, gpg_agent):
     return namespace
 
 
+def keyring_get_value(key, strict=True):
+    """    Aconvenience wrapper that attempts to get a value stored in the keyring's 'tapestry' system based on input
+    string "key". If it fails to find a value for key while kwarg strict=True, it will raise KeyError.
+    """
+    value = keyring.get_password("tapestry", key)
+    if strict:
+        if value is None:
+            raise KeyError
+    return value
+
+
 def get_user_input(message, dict_data, resp_column, list_column_order):
     # get the size of the terminal window
     width, height = shutil.get_terminal_size((80, 24))
     # make an index to add to the dict_data
     list_pass_columns = ["index"]
-    for column in list_column_order:
+    for column in list_column_order: # Todo did this ever work? Is this untested?
         list_pass_columns.append()
     index = []
     counter = 1
@@ -702,7 +719,7 @@ def unix_pack_blocks(sizes, ops_list, namespace):
             this_task = tapestry.TaskTarBuild(tarf, fid, path, block.name)
             temp_queue.append(this_task)
         this_riff = block.meta(len(collection_blocks), sum_sizes, sum_files,
-                               str(datetime.date.today()), None, ops_list, ns.drop)
+                               str(datetime.date.today()), ns.comment_string, ops_list, ns.drop)
         this_task = tapestry.TaskTarBuild(tarf, "recovery-riff",
                                           this_riff, block.name)
         tarf_queue.put(this_task)
@@ -755,6 +772,10 @@ def parse_args(namespace):
     parser.add_argument('-c', help="absolute or relative path to the config file", action="store")
     parser.add_argument('--validate', help="Expects a path or csv string of paths describing blocks to validate.",
                         action="store")
+    parser.add_argument('--secrets', help="Runs a subroutine to update the values of secrets in the keyring",
+                                         action="store_true")
+    parser.add_argument('-n', help="note to add to the metadata for this run for your future reference",
+                        action='store', default=None)
     args = parser.parse_args()
 
     ns.rcv = args.rcv
@@ -764,6 +785,8 @@ def parse_args(namespace):
     ns.genKey = args.genKey
     ns.config_path = args.c
     ns.validation_target = args.validate
+    ns.secrets = args.secrets
+    ns.comment_string = args.n
     if ns.validation_target is not None:
         ns.demand_validate = True
     else:
@@ -830,7 +853,7 @@ def windows_pack_blocks(sizes, ops_list, namespace):
                 tf.add(path, arcname=fid, recursive=False)
             status_print(current_counter, sum_files, "Packing", None)
         this_riff = block.meta(len(collection_blocks), sum_sizes, sum_files,
-                               str(datetime.date.today()), None, ops_list, ns.drop)
+                               str(datetime.date.today()), ns.comment_string, ops_list, ns.drop)
         with tarfile.open(tarf, "a:") as tf:
             tf.add(this_riff, arcname="recovery-riff", recursive=False)
 
@@ -861,35 +884,46 @@ def parse_config(namespace):
     try:
         ns.activeFP = config.get("Environment Variables", "Expected FP")
         ns.fp = config.get("Environment Variables", "Expected FP")
-        ns.signing = config.getboolean("Environment Variables", "Sign by Default")
+        ns.signing = config.getboolean("Environment Variables", "Sign by Default", fallback=True)
         ns.sigFP = config.get("Environment Variables", "Signing FP")
-        ns.keysize = config.getint("Environment Variables", "keysize")
-        ns.compress = config.getboolean("Environment Variables", "Use Compression")
-        ns.compressLevel = config.getint("Environment Variables", "Compression Level")
+        ns.keysize = config.getint("Environment Variables", "keysize", fallback=4096)
+        ns.compress = config.getboolean("Environment Variables", "Use Compression", fallback=True)
+        ns.compressLevel = config.getint("Environment Variables", "Compression Level", fallback=False)
         ns.step = "none"
         ns.sumJobs = 0
         ns.jobsDone = 0
-        ns.modeNetwork = config.get("Network Configuration", "mode")
-        ns.addrNet = config.get("Network Configuration", "server")
-        ns.portNet = config.getint("Network Configuration", "port")
-        ns.nameNet = config.get("Network Configuration", "username")
-        ns.network_credential_type = config.get("Network Configuration", "Auth Type")
-        ns.network_credential_value = config.get("Network Configuration", "Credential Path")
-        ns.network_credential_pass = config.getboolean("Network Configuration", "Credential Has Passphrase")
-        ns.dirNet = config.get("Network Configuration", "remote drop location")
-        ns.retainLocal = config.getboolean("Network Configuration", "Keep Local Copies")
-        ns.block_size_raw = config.getint("Environment Variables", "blockSize") * (
-            2 ** 20)  # The math is necessary to go from MB to Bytes)
-        ns.compid = config.get("Environment Variables", "compid")
-        ns.recovery_path = config.get("Environment Variables", "recovery path")
+        ns.modeNetwork = config.get("Network Configuration", "mode", fallback="none")
+        ns.addrNet = config.get("Network Configuration", "server", fallback=None)
+        ns.portNet = config.getint("Network Configuration", "port", fallback=None)
+        ns.nameNet = config.get("Network Configuration", "username", fallback=None)
+        ns.network_credential_type = config.get("Network Configuration", "Auth Type", fallback=None)
+        ns.network_credential_value = config.get("Network Configuration", "Credential Path", fallback=None)
+        ns.network_credential_pass = config.getboolean("Network Configuration", "Credential Has Passphrase", fallback=None)
+        if ns.network_credential_type.lower() == "passphrase":
+            ns.network_credential_value = keyring_get_value("network_passphrase")
+        if ns.network_credential_pass:
+            ns.network_credential_value = keyring_get_value("network_passphrase")
+        if ns.rcv:
+            ns.encryption_key_passphrase = keyring_get_value("decryption_passphrase", strict=False)
+        ns.signing_key_passphrase = keyring_get_value("signing_passphrase", strict=False)
+        ns.dirNet = config.get("Network Configuration", "remote drop location", fallback=None)
+        ns.retainLocal = config.getboolean("Network Configuration", "Keep Local Copies", fallback=True)
+        ns.block_size_raw = config.getint("Environment Variables", "blockSize", fallback=2000) * (
+            2 ** 20)  # The math is necessary to go from MB to Bytes
+        ns.compid = config.get("Environment Variables", "compid", fallback="Tapestry")
+        ns.recovery_path = config.get("Environment Variables", "recovery path", fallback=None)
         ns.uid = config.get("Environment Variables", "uid")
-        ns.drop = config.get("Environment Variables", "Output Path")
-        ns.do_validation = config.getboolean("Environment Variables", "Build-Time File Validation")
+        ns.drop = config.get("Environment Variables", "Output Path", fallback=None)
+        ns.do_validation = config.getboolean("Environment Variables", "Build-Time File Validation", fallback=True)
     except configparser.NoOptionError:
-        print("Tapestry has attempted to reference a configuration option which is missing from the config file.")
+        print("Tapestry has attempted to reference a required option which is missing from the config file.")
         print("Please confirm the structure of your config file is correct.")
-        print("If you have just upgraded to 2.1 or later, you may be missing the network options fields.")
+        print("If you have just upgraded to 2.2 or later, you may be missing added fields.")
         exit(3)
+    except KeyError:
+        print("Tapestry was unable to find a value it expected in the keyring for config flag %s.")
+        print("Please rerun tapestry with the flag --secrets specified to adjust these values.")
+        exit(6)
 
     if ns.currentOS == "Linux":
         ns.workDir = "/tmp/Tapestry/"
@@ -1131,6 +1165,18 @@ def unpack_blocks(namespace):
     tasks.join()
 
 
+def update_secrets():
+    print("You have selected --secrets, and we will now go through the possible secret values to encode them.")
+    print("Secrets stored in this way are stored in your system keyring and not the tapestry system files.")
+    print("Python will have access to these fields! If a secret value is not desired to be stored, press enter to skip")
+    print("A blank value will delete the current stored value for that key for security reasons.")
+    print("Check the documentation to see how each value is used.")
+    secret_keys = ["network_passphrase", "signing_passphrase", "decryption_passphrase"]
+    for each in secret_keys:
+        value = getpass.getpass(prompt="%s?" % each)
+        keyring.set_password("tapestry", each, value)
+
+
 def verify_blocks(ns, gpg_agent, testing=False):
     """Verifies blocks and returns a list of verified blocks as a result"""
     gpg = gpg_agent
@@ -1189,14 +1235,19 @@ def verify_keys(ns, gpg):
 
 def runtime():
     global state
+    keyring.get_keyring()
     state = Namespace()
     state = parse_args(state)
+    if state.secrets:
+        update_secrets()
+        exit(0)
     state = parse_config(state)
     state = start_logging(state)
     gpg_conn = start_gpg(state)
     announce()
     if state.modeNetwork.lower() != "none":
-        if state.network_credential_pass or (state.network_credential_type == "passphrase"):
+        if (state.network_credential_pass or (state.network_credential_type == "passphrase"))\
+                and state.network_credential_value is None:
             print("Tapestry is running in network storage mode and requires some credentialing information.")
             state.network_credential_pass = getpass.getpass(prompt="Enter Network Credential Passphrase: ")
     if state.demand_validate:
